@@ -165,17 +165,17 @@ static TYPE_FD sock_open(const char *fn)
 #else /* WIN32 */
 
 	fd = CreateNamedPipe(
-			fn,			// pipe name
-			PIPE_ACCESS_DUPLEX |  // read/write access
-			FILE_FLAG_OVERLAPPED, // async IO
+			fn,			/* pipe name */
+			PIPE_ACCESS_DUPLEX |	/* read/write access */
+			FILE_FLAG_OVERLAPPED,	/* async IO */
 			PIPE_TYPE_BYTE |
 			PIPE_READMODE_BYTE |
 			PIPE_WAIT,
-			PIPE_UNLIMITED_INSTANCES, // max. instances
-			ST_SOCK_BUF_LEN,	// output buffer size
-			ST_SOCK_BUF_LEN,	// input buffer size
-			0,			// client time-out
-			NULL);			// FIXME: default security attribute
+			PIPE_UNLIMITED_INSTANCES,	/* max. instances */
+			ST_SOCK_BUF_LEN,	/* output buffer size */
+			ST_SOCK_BUF_LEN,	/* input buffer size */
+			0,			/* client time-out */
+			NULL);			/* FIXME: default security attribute */
 
 	if (INVALID_FD(fd)) {
 		fatal_with_errno(EXIT_FAILURE,
@@ -507,7 +507,7 @@ static void sock_connect(TYPE_FD sock)
 		upsdebugx(0, "%s: keeping default synchronous mode", __func__);
 	}
 
-	conn = xcalloc(1, sizeof(*conn));
+	conn = (conn_t *)xcalloc(1, sizeof(*conn));
 	conn->fd = fd;
 
 #else /* WIN32 */
@@ -520,17 +520,17 @@ static void sock_connect(TYPE_FD sock)
 
 	/* sockfd is the handle of the connection pending pipe */
 	sockfd = CreateNamedPipe(
-			pipename,			// pipe name
-			PIPE_ACCESS_DUPLEX |  // read/write access
-			FILE_FLAG_OVERLAPPED, // async IO
+			pipename,		/* pipe name */
+			PIPE_ACCESS_DUPLEX |	/* read/write access */
+			FILE_FLAG_OVERLAPPED,	/* async IO */
 			PIPE_TYPE_BYTE |
 			PIPE_READMODE_BYTE |
 			PIPE_WAIT,
-			PIPE_UNLIMITED_INSTANCES, // max. instances
-			ST_SOCK_BUF_LEN,	// output buffer size
-			ST_SOCK_BUF_LEN,	// input buffer size
-			0,			// client time-out
-			NULL);			// FIXME: default security attribute
+			PIPE_UNLIMITED_INSTANCES,	/* max. instances */
+			ST_SOCK_BUF_LEN,	/* output buffer size */
+			ST_SOCK_BUF_LEN,	/* input buffer size */
+			0,			/* client time-out */
+			NULL);			/* FIXME: default security attribute */
 
 	if (INVALID_FD(sockfd)) {
 		fatal_with_errno(EXIT_FAILURE,
@@ -780,7 +780,7 @@ static int sock_arg(conn_t *conn, size_t numarg, char **arg)
 			upsdebugx(3, "%s: TRACKING = %s", __func__, cmdid);
 
 		/* try the handler shared by all drivers first */
-		ret = main_instcmd(arg[1], arg[2], conn);
+		ret = main_instcmd(cmdname, cmdparam, conn);
 		if (ret != STAT_INSTCMD_UNKNOWN) {
 			/* The command was acknowledged by shared handler, and
 			 * either handled successfully, or failed, or was not
@@ -1675,9 +1675,15 @@ void alarm_set(const char *buf)
 # pragma GCC diagnostic pop
 #endif
 
-/* write the status_buf into the info array */
+/* write the status_buf into the info array for "ups.alarm" */
 void alarm_commit(void)
 {
+	/* Note this is a bit different from `device_alarm_commit(0);`
+	 * because here we also increase AND zero out the alarm count.
+	 *		alarm_active = 0; device_alarm_commit(0);
+	 * would be equivalent, but too intimate for later maintenance.
+	 */
+
 	if (strlen(alarm_buf) > 0) {
 		dstate_setinfo("ups.alarm", "%s", alarm_buf);
 		alarm_active = 1;
@@ -1694,6 +1700,8 @@ void device_alarm_init(void)
 }
 
 /* same as above, but writes to "device.X.ups.alarm" or "ups.alarm" */
+/* Note that 20 chars below just allow for a 2-digit "X" */
+/* FIXME? Shouldn't this be changed to be a LARGEBUF aka sizeof(alarm_buf) ? */
 void device_alarm_commit(const int device_number)
 {
 	char info_name[20];
