@@ -351,6 +351,36 @@ void storeval(const char *var, char *val)
 	printf("Look in the man page or call this driver with -h for a list of\n");
 	printf("valid variable names and flags.\n");
 
+	if (!strcmp(progname, "nutdrv_qx")) {
+		/* First many entries are from nut_usb_addvars() implementations;
+		 * the latter two (about langid) are from nutdrv_qx.c
+		 */
+		if (!strcmp(var, "vendor")
+		||  !strcmp(var, "product")
+		||  !strcmp(var, "serial")
+		||  !strcmp(var, "vendorid")
+		||  !strcmp(var, "productid")
+		||  !strcmp(var, "bus")
+		||  !strcmp(var, "device")
+		||  !strcmp(var, "busport")
+		||  !strcmp(var, "usb_set_altinterface")
+		||  !strcmp(var, "usb_config_index")
+		||  !strcmp(var, "usb_hid_rep_index")
+		||  !strcmp(var, "usb_hid_desc_index")
+		||  !strcmp(var, "usb_hid_ep_in")
+		||  !strcmp(var, "usb_hid_ep_out")
+		||  !strcmp(var, "allow_duplicates")
+		||  !strcmp(var, "langid_fix")
+		||  !strcmp(var, "noscanlangid")
+		) {
+			printf("\nNOTE: for driver '%s', options like '%s' are only available\n"
+				"if it was built with USB support. If you are running a custom build of NUT,\n"
+				"please check results of the `configure` checks, and consider an explicit\n"
+				"`--with-usb` option. Also make sure that both libusb library and headers\n"
+				"are installed in your build environment.\n\n", progname, var);
+		}
+	}
+
 	exit(EXIT_SUCCESS);
 }
 
@@ -1004,7 +1034,7 @@ static int main_arg(char *var, char *val)
 	 * catch commented-away settings, so not checking previous value.
 	 */
 	if (!strcmp(var, "debug_min")) {
-		int lvl = -1; // typeof common/common.c: int nut_debug_level
+		int lvl = -1; /* typeof common/common.c: int nut_debug_level */
 		if ( str_to_int (val, &lvl, 10) && lvl >= 0 ) {
 			nut_debug_level_driver = lvl;
 		} else {
@@ -1129,7 +1159,7 @@ static void do_global_args(const char *var, const char *val)
 	 * catch commented-away settings, so not checking previous value.
 	 */
 	if (!strcmp(var, "debug_min")) {
-		int lvl = -1; // typeof common/common.c: int nut_debug_level
+		int lvl = -1; /* typeof common/common.c: int nut_debug_level */
 		if ( str_to_int (val, &lvl, 10) && lvl >= 0 ) {
 			nut_debug_level_global = lvl;
 		} else {
@@ -1616,9 +1646,64 @@ int main(int argc, char **argv)
 	const char * cmd = NULL;
 #endif
 
+	const char optstring[] = "+a:s:kDFBd:hx:Lqr:u:g:Vi:c:"
+#ifndef WIN32
+		"P:"
+#endif
+		;
+
 	/* init verbosity from default in common.c (0 probably) */
 	nut_debug_level_args = nut_debug_level;
 
+	/* handle CLI-driven debug level in advance, to trace initialization if needed */
+	while ((i = getopt(argc, argv, optstring)) != -1) {
+		switch (i) {
+			case 'D':
+				/* bump right here, may impact reporting of other CLI args */
+				nut_debug_level++;
+				nut_debug_level_args++;
+				break;
+		}
+	}
+	/* Reset the index, read argv[1] next time (loop below)
+	 * https://pubs.opengroup.org/onlinepubs/9699919799/functions/getopt.html
+	 */
+	optind = 1;
+
+	if (foreground < 0) {
+		/* Guess a default */
+		/* Note: only care about CLI-requested debug verbosity here */
+		if (nut_debug_level > 0 || dump_data) {
+			/* Only flop from default - stay foreground with debug on */
+			foreground = 1;
+		} else {
+			/* Legacy default - stay background and quiet */
+			foreground = 0;
+		}
+	} else {
+		/* Follow explicit user -F/-B request */
+		upsdebugx (0,
+			"Debug level is %d, dump data count is %s, "
+			"but backgrounding mode requested as %s",
+			nut_debug_level,
+			dump_data ? "on" : "off",
+			foreground ? "off" : "on"
+			);
+	}
+
+	{ /* scoping */
+		char *s = getenv("NUT_DEBUG_LEVEL");
+		int l;
+		if (s && str_to_int(s, &l, 10)) {
+			if (l > 0 && nut_debug_level_args < 1) {
+				upslogx(LOG_INFO, "Defaulting debug verbosity to NUT_DEBUG_LEVEL=%d "
+					"since none was requested by command-line options", l);
+				nut_debug_level = l;
+				nut_debug_level_args = l;
+			}	/* else follow -D settings */
+		}	/* else nothing to bother about */
+	}
+
 	dstate_setinfo("driver.state", "init.starting");
 
 	atexit(exit_cleanup);
@@ -1660,11 +1745,7 @@ int main(int argc, char **argv)
 	/* build the driver's extra (-x) variable table */
 	upsdrv_makevartable();
 
-	while ((i = getopt(argc, argv, "+a:s:kFBDd:hx:Lqr:u:g:Vi:c:"
-#ifndef WIN32
-		"P:"
-#endif
-	)) != -1) {
+	while ((i = getopt(argc, argv, optstring)) != -1) {
 		switch (i) {
 			case 'a':
 				if (upsname)
@@ -1699,9 +1780,7 @@ int main(int argc, char **argv)
 				foreground = 0;
 				break;
 			case 'D':
-				/* bump right here, may impact reporting of other CLI args */
-				nut_debug_level++;
-				nut_debug_level_args++;
+				/* Processed above */
 				break;
 			case 'd':
 				dump_data = atoi(optarg);
@@ -1828,40 +1907,6 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (foreground < 0) {
-		/* Guess a default */
-		/* Note: only care about CLI-requested debug verbosity here */
-		if (nut_debug_level > 0 || dump_data) {
-			/* Only flop from default - stay foreground with debug on */
-			foreground = 1;
-		} else {
-			/* Legacy default - stay background and quiet */
-			foreground = 0;
-		}
-	} else {
-		/* Follow explicit user -F/-B request */
-		upsdebugx (0,
-			"Debug level is %d, dump data count is %s, "
-			"but backgrounding mode requested as %s",
-			nut_debug_level,
-			dump_data ? "on" : "off",
-			foreground ? "off" : "on"
-			);
-	}
-
-	{ /* scoping */
-		char *s = getenv("NUT_DEBUG_LEVEL");
-		int l;
-		if (s && str_to_int(s, &l, 10)) {
-			if (l > 0 && nut_debug_level_args < 1) {
-				upslogx(LOG_INFO, "Defaulting debug verbosity to NUT_DEBUG_LEVEL=%d "
-					"since none was requested by command-line options", l);
-				nut_debug_level = l;
-				nut_debug_level_args = l;
-			}	/* else follow -D settings */
-		}	/* else nothing to bother about */
-	}
-
 	/* Since debug mode dumps from drivers are often posted to mailing list
 	 * or issue tracker, as well as viewed locally, it can help to know the
 	 * build options involved when troubleshooting (especially when needed
@@ -2260,6 +2305,9 @@ int main(int argc, char **argv)
 			/* Use file descriptor, not name, to first check and then manipulate permissions:
 			 *   https://cwe.mitre.org/data/definitions/367.html
 			 *   https://wiki.sei.cmu.edu/confluence/display/c/FIO01-C.+Be+careful+using+functions+that+use+file+names+for+identification
+			 * Alas, Unix sockets on most systems can not be open()ed
+			 * so there is no file descriptor to manipulate.
+			 * Fall back to name-based "les secure" operations then.
 			 */
 			TYPE_FD fd = ERROR_FD;
 
@@ -2275,8 +2323,8 @@ int main(int argc, char **argv)
 
 			if (grp == NULL) {
 				upsdebugx(1, "WARNING: could not resolve "
-					"group name '%s': %s",
-					group, strerror(errno)
+					"group name '%s' (%i): %s",
+					group, errno, strerror(errno)
 				);
 				allOk = 0;
 				goto sockname_ownership_finished;
@@ -2285,35 +2333,46 @@ int main(int argc, char **argv)
 				mode_t mode;
 
 				if (INVALID_FD((fd = open(sockname, O_RDWR | O_APPEND)))) {
-					upsdebugx(1, "WARNING: opening socket file for stat/chown failed: %s",
-						strerror(errno)
+					upsdebugx(1, "WARNING: opening socket file for stat/chown failed "
+						"(%i), which is rather typical for Unix socket handling: %s",
+						errno, strerror(errno)
 					);
 					allOk = 0;
-					/* Can not proceed with ops below */
-					goto sockname_ownership_finished;
 				}
 
-				if (fstat(fd, &statbuf)) {
-					upsdebugx(1, "WARNING: stat for chown failed: %s",
-						strerror(errno)
+				if ((VALID_FD(fd) && fstat(fd, &statbuf))
+				||  (INVALID_FD(fd) && stat(sockname, &statbuf))
+				) {
+					upsdebugx(1, "WARNING: stat for chown of socket file failed (%i): %s",
+						errno, strerror(errno)
 					);
 					allOk = 0;
+					if (INVALID_FD(fd)) {
+						/* Can not proceed with ops below */
+						goto sockname_ownership_finished;
+					}
 				} else {
+					/* Maybe open() and some stat() succeeed so far */
+					allOk = 1;
 					/* Here we do a portable chgrp() essentially: */
-					if (fchown(fd, statbuf.st_uid, grp->gr_gid)) {
-						upsdebugx(1, "WARNING: chown failed: %s",
-							strerror(errno)
+					if ((VALID_FD(fd) && fchown(fd, statbuf.st_uid, grp->gr_gid))
+					||  (INVALID_FD(fd) && chown(sockname, statbuf.st_uid, grp->gr_gid))
+					) {
+						upsdebugx(1, "WARNING: chown of socket file failed (%i): %s",
+							errno, strerror(errno)
 						);
 						allOk = 0;
 					}
 				}
 
 				/* Refresh file info */
-				if (fstat(fd, &statbuf)) {
+				if ((VALID_FD(fd) && fstat(fd, &statbuf))
+				||  (INVALID_FD(fd) && stat(sockname, &statbuf))
+				) {
 					/* Logically we'd fail chown above if file
 					 * does not exist or is not accessible */
-					upsdebugx(1, "WARNING: stat for chmod failed: %s",
-						strerror(errno)
+					upsdebugx(1, "WARNING: stat for chmod of socket file failed (%i): %s",
+						errno, strerror(errno)
 					);
 					allOk = 0;
 				} else {
@@ -2321,9 +2380,11 @@ int main(int argc, char **argv)
 					mode = statbuf.st_mode;
 					mode |= S_IWGRP;
 					mode |= S_IRGRP;
-					if (fchmod(fd, mode)) {
-						upsdebugx(1, "WARNING: chmod failed: %s",
-							strerror(errno)
+					if ((VALID_FD(fd) && fchmod(fd, mode))
+					|| (INVALID_FD(fd) && chmod(sockname, mode))
+					) {
+						upsdebugx(1, "WARNING: chmod of socket file failed (%i): %s",
+							errno, strerror(errno)
 						);
 						allOk = 0;
 					}
@@ -2331,13 +2392,10 @@ int main(int argc, char **argv)
 			}
 
 sockname_ownership_finished:
-			if (VALID_FD(fd)) {
-				close(fd);
-				fd = ERROR_FD;
-			}
-
 			if (allOk) {
-				upsdebugx(1, "Group access for this driver successfully fixed");
+				upsdebugx(1, "Group access for this driver successfully fixed "
+					"(using file %s based methods)",
+					VALID_FD(fd) ? "descriptor" : "name");
 			} else {
 				upsdebugx(0, "WARNING: Needed to fix group access "
 					"to filesystem socket of this driver, but failed; "
@@ -2347,6 +2405,11 @@ sockname_ownership_finished:
 					"the device: %s",
 					sockname);
 			}
+
+			if (VALID_FD(fd)) {
+				close(fd);
+				fd = ERROR_FD;
+			}
 #else	/* not WIN32 */
 			upsdebugx(1, "Options for alternate user/group are not implemented on this platform");
 #endif	/* WIN32 */
