@@ -430,7 +430,7 @@ static ssize_t powpan_status(status_t *status)
 	/*
 	 * WRITE D\r
 	 * READ #VVL.CTF.....\r
-        *      01234567890123
+	 *      01234567890123
 	 */
 	ret = ser_send_pace(upsfd, UPSDELAY, "D\r");
 
@@ -545,7 +545,7 @@ static int powpan_updateinfo(void)
 	}
 
 	/* !OB && !TEST */
-	if (!(status.flags[0] & 0x84)) {
+	if (!(status.flags[0] & 0x84) && status.o_volt) {
 
 		if (status.o_volt < 0.5 * status.i_volt) {
 			upsdebugx(2, "%s: output voltage too low", __func__);
