@@ -230,7 +230,7 @@ err:
 /* end of dynamic link library stuff */
 
 static char* is_usb_device_supported(usb_device_id_t *usb_device_id_list,
-					int dev_VendorID, int dev_ProductID)
+					int dev_VendorID, int dev_ProductID, char **alt)
 {
 	usb_device_id_t *usbdev;
 
@@ -238,6 +238,8 @@ static char* is_usb_device_supported(usb_device_id_t *usb_device_id_list,
 		if ((usbdev->vendorID == dev_VendorID)
 		 && (usbdev->productID == dev_ProductID)
 		) {
+			if (alt)
+				*alt = usbdev->alt_driver_names;
 			return usbdev->driver_name;
 		}
 	}
@@ -246,7 +248,7 @@ static char* is_usb_device_supported(usb_device_id_t *usb_device_id_list,
 }
 
 /* return NULL if error */
-nutscan_device_t * nutscan_scan_usb(void)
+nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 {
 	int ret;
 	char string[256];
@@ -256,6 +258,7 @@ nutscan_device_t * nutscan_scan_usb(void)
 	 * (drivers/usb-common.h).
 	 */
 	char *driver_name = NULL;
+	char *alt_driver_names = NULL;
 	char *serialnumber = NULL;
 	char *device_name = NULL;
 	char *vendor_name = NULL;
@@ -277,8 +280,12 @@ nutscan_device_t * nutscan_scan_usb(void)
 	 * In libusb0 API: "device filename"
 	 */
 	char *device_port = NULL;
+
 	/* bcdDevice: aka "Device release number" - note we currently do not match by it */
 	uint16_t bcdDevice = 0;
+
+	nutscan_usb_t	default_scanopts;
+
 #if WITH_LIBUSB_1_0
 	libusb_device *dev;
 	libusb_device **devlist;
@@ -305,6 +312,18 @@ nutscan_device_t * nutscan_scan_usb(void)
 		return NULL;
 	}
 
+	if (!scanopts) {
+		default_scanopts.report_bus = 1;
+		default_scanopts.report_busport = 1;
+		/* AQU note: disabled by default, since it may lead to instabilities
+		 * and give more issues than solutions! */
+		default_scanopts.report_device = 0;
+		/* Generally not useful at the moment, and coded to be commented away
+		 * in formats that support it (e.g. ups.conf) or absent in others. */
+		default_scanopts.report_bcdDevice = 0;
+		scanopts = &default_scanopts;
+	}
+
 	/* libusb base init */
 	/* Initialize Libusb */
 #if WITH_LIBUSB_1_0
@@ -397,7 +416,7 @@ nutscan_device_t * nutscan_scan_usb(void)
 #endif
 			if ((driver_name =
 				is_usb_device_supported(usb_device_table,
-					VendorID, ProductID)) != NULL) {
+					VendorID, ProductID, &alt_driver_names)) != NULL) {
 
 				/* open the device */
 #if WITH_LIBUSB_1_0
@@ -533,6 +552,9 @@ nutscan_device_t * nutscan_scan_usb(void)
 				if (driver_name) {
 					nut_dev->driver = strdup(driver_name);
 				}
+				if (alt_driver_names) {
+					nut_dev->alt_driver_names = strdup(alt_driver_names);
+				}
 				nut_dev->port = strdup("auto");
 
 				sprintf(string, "%04X", VendorID);
@@ -569,29 +591,38 @@ nutscan_device_t * nutscan_scan_usb(void)
 					vendor_name = NULL;
 				}
 
-				nutscan_add_option_to_device(nut_dev,
+				nutscan_add_commented_option_to_device(nut_dev,
 					"bus",
-					busname);
+					busname,
+					scanopts->report_bus ? NULL : "");
 
-				nutscan_add_option_to_device(nut_dev,
+				nutscan_add_commented_option_to_device(nut_dev,
 					"device",
-					device_port);
+					device_port,
+					scanopts->report_device ? NULL : "");
 
 #if WITH_LIBUSB_1_0
 				if (bus_port) {
-					nutscan_add_option_to_device(nut_dev,
+					nutscan_add_commented_option_to_device(nut_dev,
 						"busport",
-						bus_port);
+						bus_port,
+						scanopts->report_busport ? NULL : "");
 					free(bus_port);
 					bus_port = NULL;
 				}
 #endif	/* WITH_LIBUSB_1_0 */
 
-				/* Not currently matched by drivers, hence commented for now: */
-				sprintf(string, "%04X", bcdDevice);
-				nutscan_add_option_to_device(nut_dev,
-					"###NOTMATCHED-YET###bcdDevice",
-					string);
+				/* FIXME: Detect and suggest HID index, interface number, etc. */
+
+				if (scanopts->report_bcdDevice) {
+					/* Not currently matched by drivers, hence commented
+					 * for now even if requested via scanopts */
+					sprintf(string, "%04X", bcdDevice);
+					nutscan_add_commented_option_to_device(nut_dev,
+						"bcdDevice",
+						string,
+						"NOTMATCHED-YET");
+				}
 
 				current_nut_dev = nutscan_add_device_to_device(
 					current_nut_dev,
@@ -623,8 +654,10 @@ nutscan_device_t * nutscan_scan_usb(void)
 #else /* not WITH_USB */
 
 /* stub function */
-nutscan_device_t * nutscan_scan_usb(void)
+nutscan_device_t * nutscan_scan_usb(nutscan_usb_t * scanopts)
 {
+	NUT_UNUSED_VARIABLE(scanopts);
+
 	return NULL;
 }
 
